from flask import Flask, render_template

import cal.utils as utils
import cal.sections as sections
from cal.latex import Latex
import os
from pprint import pprint

class Trans:
    gas_trans = {
        "N2":{
            "de" :"Stickstoff",
            "en": "nitrogen",
            "mol_weigth":"28.013"
        },
        "He":{
            "de":"Helium",
            "en": "helium",
            "mol_weigth":"4.0026"
        },
        "Ar":{
            "de":"Argon",
            "en":"argon",
            "mol_weigth":"39.948"
        },
        "H2":{
            "de":"Wasserstoff",
            "en":"hydrogen",
            "mol_weigth":"2.016"
        },        
        "Kr":{
            "de":"Krypton",
            "en":"krypton",
            "mol_weigth":"83.8"
        },
        "Ne":{
            "de":"Neon",
            "en":"neon",
            "mol_weigth":"20.179"
        }

    }
    meas_type_trans = {
        "cont_expansion":{
            "de" :", Kalibrierung nach dem kontinuierlichen Expansionsverfahren",
            "en": ", continuous expansion method"
        },
        "sens":{
            "de" :", Kalibrierung nach dem kontinuierlichen Expansionsverfahren",
            "en": ", continuous expansion method"
        },
        "srg_vg":{
            "de" :", Kalibrierung durch SRG Vergleich",
            "en": ", SRG comparison method"
        },

        "expansion":{
            "de" :", Kalibrierung nach dem statischen Verfahren",
            "en": ", static expansion method"
        },
        "se2_expansion":{
            "de" :"",
            "en": ""
        },
        "se2_expansion_direct":{
            "de" :"",
            "en": ""
        },
        "se2_expansion":{
            "de" :"",
            "en": ""
        },
        "direct":{
            "de": ", Kalibrierung durch direkten Vergleich",
            "en": ", direct comparison method"
        },
        "pressure_balance":{
            "de": ", Kalibrierung durch Vergleich mit Druckwaage",
            "en": ", comparison with pressure balance"
        },
        "rotary_piston_gauge":{
            "de": ", Kalibrierung durch Vergleich mit Drehkolbenmanometer",
            "en": ", comparison with rotary piston gauge"
        }
    }



    def __init__(self, config, app, db):
        self.latex_path = config['templates']['latex']
        self.texenv = Latex(app)
        self.db = db

    def cert_to_latex(self, cert_doc):
        latex_template = utils.path_file(path=self.latex_path, file='certificate.tex')
        template = self.texenv.env.get_template(latex_template)
        if 'Table' in cert_doc['Certificate']:
            table = self.texenv.make_table(cert_doc['Certificate']['Table'])
            cert_doc['Certificate']['Table'] = table

        return template.render(doc=cert_doc['Certificate'])

    def cert(self, cal_docs):
        maintainers =  self.db.get_maintainers()
        countries = self.db.get_countries()
        cert = {
            "RequestId": utils.get_request_id(cal_docs=cal_docs),
            "Meta": utils.get_meta(cal_docs=cal_docs),
            "Titlepage": utils.get_titlepage(cal_docs=cal_docs, countries=countries, maintainers=maintainers),
            "Device": utils.get_device(cal_docs=cal_docs),
            "Helper":{"gas_trans":self.gas_trans, "meas_type_trans":self.meas_type_trans},
            "Table": utils.get_tables(cal_docs=cal_docs),
            "Formula": utils.get_formulars(cal_docs=cal_docs),
            "MeasurementData":  utils.get_measurement_data(cal_docs=cal_docs)}

        eakte = utils.get_eakte(cal_docs=cal_docs)
        if eakte:
            cert["EAkte"] = eakte

        secs = sections.get_all(cert=cert)
        for i, sec in enumerate(secs):
            h = sec.get("Heading")
            template = self.texenv.env.from_string(h)
            sec["Heading"] = template.render(cert).strip()
            for j, para in enumerate(sec.get("Paragraph")):
                p = para.get("Text")
                template = self.texenv.env.from_string(p)
                para["Text"] = template.render(cert).strip()
        cert["Section"] = secs

        return cert

    def dcc(self, cal_docs):
        cal_docs = [doc for doc in cal_docs if not utils.should_skipped(doc)]

        maintainers =  self.db.get_maintainers()
        countries = self.db.get_countries()
        info_doc = cal_docs[-1]
        lang = utils.get_lang(info_doc)
        begin_meas_date, end_meas_date = utils.meas_date_begin_end(cal_docs)

        cert = {"RequestId": utils.get_request_id(cal_docs=cal_docs),
                "Meta": utils.get_meta(cal_docs=cal_docs),
                "Device": utils.get_device(cal_docs=cal_docs),
                "Helper":{"gas_trans":self.gas_trans, "meas_type_trans":self.meas_type_trans},
                "Table": utils.get_tables(cal_docs=cal_docs),
                "MeasurementData":  utils.get_measurement_data(cal_docs=cal_docs),}

        secs = sections.get_all(cert)
        ## needed for replacement of second order templates
        ## (like Device.Text.shipping etc)
        for i, sec in enumerate(secs):
            h = sec.get("Heading")
            template = self.texenv.env.from_string(h)
            heading = sections.rm(template.render(cert).strip())
            sec["Heading"] = heading
            for j, para in enumerate(sec.get("Paragraph")):
                p = para.get("Text")
                template = self.texenv.env.from_string(p)
                para["Text"] = template.render(cert).strip()

        return {"administrativeData":{"coreData":{"countryCodeISO3166_1":"DE",
                                                  "usedLangCodeISO639_1":"de",
                                                  "usedLangCodeISO639_1":"en",
                                                  "mandatoryLangCodeISO639_1":lang,
                                                  "uniqueIdentifier": utils.dcc_unique_identifier(info_doc),
                                                  "identifications": utils.dcc_identification_lab(info_doc),
                                                  #"receiptDate": utils.get_current_date(),
                                                  "beginPerformanceDate": begin_meas_date,
                                                  "endPerformanceDate": end_meas_date,
                                                  "performanceLocation": "laboratory"},
                                      "items":{"name":[{"lang":lang, "content":[utils.get_customerobject_type(info_doc)]}],
                                               "item":[{"identifications": utils.dcc_identification_manufacturer(info_doc),
                                                        "description":utils.dcc_calibration_object_description(cal_docs, secs),
                                                        "name":[{"lang": utils.get_lang(info_doc), "content": [utils.get_customerobject_device_type(info_doc)]}],
                                                        "manufacturer": utils.dcc_manufacturer(info_doc)}]},
                                      "calibrationLaboratory": utils.dcc_calibration_laboratory(),
                                      "respPersons":{"respPerson":[utils.dcc_resp_person(cal_docs, maintainers),
                                                                   utils.dcc_meas_maintainer(cal_docs, maintainers)]},
                                      "customer": utils.dcc_customer(cal_docs, countries),
                                      "dccSoftware":utils.dcc_software(cal_docs),
                                      "statements": utils.dcc_statements(cal_docs)},
                "measurementResults":{"name":[{"lang":"de", "content":["Ergebnis der Kalibrierung"]},
                                              {"lang":"en", "content":["Result of the calibration"]}],
                                      "usedMethods":{"usedMethod": utils.dcc_used_methodes(cal_docs, secs)},
                                      "influenceConditions": utils.dcc_influence_cond(cal_docs,
                                                                                      self.gas_trans,
                                                                                      self.meas_type_trans,
                                                                                      utils.dcc_unit_trans()),
                                      "results": {"result":utils.dcc_result(cal_docs,
                                                                            self.gas_trans,
                                                                            self.meas_type_trans,
                                                                            utils.dcc_unit_trans())}}}
